@@ -2,29 +2,33 @@
 UBX Protocol Output payload definitions
 
 THESE ARE THE PAYLOAD DEFINITIONS FOR _GET_ MESSAGES _FROM_ THE RECEIVER
-(i.e. Periodic Navigation Data, Poll Responses, Info messages)
+(e.g. Periodic Navigation Data; Poll Responses; Info messages)
 
 NB: Attribute names must be unique within each message class/id
 
-NB: Repeating groups must be defined as a tuple thus
+NB: Repeating or bitfield groups must be defined as a tuple thus
     'group': ('numr', {dict})
     where
     - 'numr' is either:
        a) an integer representing a fixed number of repeats e.g 32
        b) a string representing the name of a preceding attribute
           containing the number of repeats e.g. 'numCh'
-       c) 'None' for a 'variable by size' repeating group
+       c) an 'X' attribute type ('X1', 'X2', 'X4', etc) representing a group of individual bit flags
+       d) 'None' for a 'variable by size' repeating group
           (only one such group is permitted per message type)
     - {dict} is the nested dictionary containing the repeating
       attributes
 
 Created on 27 Sep 2020
 
+Information sourced from u-blox Interface Specifications © 2013-2021, u-blox AG
+
 :author: semuadmin
 """
 # pylint: disable=too-many-lines, line-too-long
 
 from pyubx2.ubxtypes_core import (
+    A256,
     C2,
     C6,
     C10,
@@ -46,6 +50,7 @@ from pyubx2.ubxtypes_core import (
     U8,
     U9,
     U12,
+    U22,
     U64,
     X1,
     X2,
@@ -135,16 +140,60 @@ UBX_PAYLOADS_GET = {
     # Messages in the CFG class are used to configure the receiver and read out current configuration values. Any
     # messages in the CFG class sent to the receiver are either acknowledged (with message UBX-ACK-ACK) if
     # processed successfully or rejected (with message UBX-ACK-NAK) if processing unsuccessfully.
-    "CFG-ANT": {"flags": X2, "pins": X2},
+    "CFG-ANT": {
+        "flags": (
+            X2,
+            {
+                "svcs": U1,
+                "scd": U1,
+                "ocd": U1,
+                "pdwnOnSCD": U1,
+                "recovery": U1,
+            },
+        ),
+        "pins": (
+            X2,
+            {
+                "pinSwitch": U5,
+                "pinSCD": U5,
+                "pinOCD": U5,
+                "reconfig": U1,
+            },
+        ),
+    },
     "CFG-BATCH": {
         "version": U1,
-        "flags": X1,
+        "flags": (
+            X1,
+            {
+                "enable": U1,
+                "reserved1": U1,
+                "extraPvt": U1,
+                "extraOdo": U1,
+                "reserved2": U1,
+                "pioEnable": U1,
+                "pioActiveLow": U1,
+            },
+        ),
         "bufSize": U2,
         "notifThrs": U2,
         "pioId": U1,
         "reserved0": U1,
     },
-    "CFG-CFG": {"clearMask": X4, "saveMask": X4, "loadMask": X4, "deviceMask": X1},
+    "CFG-CFG": {
+        "clearMask": X4,
+        "saveMask": X4,
+        "loadMask": X4,
+        "deviceMask": (
+            X1,
+            {
+                "devBBR": U1,
+                "devFlash": U1,
+                "devEEPROM": U1,
+                "devSpiFlash": U1,
+            },
+        ),
+    },
     "CFG-DAT": {
         "datumNum": U2,
         "datumName": C6,
@@ -171,7 +220,13 @@ UBX_PAYLOADS_GET = {
             {  # repeating group * numOsc
                 "oscId": U1,
                 "reserved2": U1,
-                "flags": X2,
+                "flags": (
+                    X2,
+                    {
+                        "isCalibrated": U1,
+                        "controlIf": U4,
+                    },
+                ),
                 "freq": U4,
                 "phaseOffset": I4,
                 "withTemp": U4,
@@ -212,15 +267,38 @@ UBX_PAYLOADS_GET = {
     },
     "CFG-ESFWT": {
         "version": U1,
-        "flags1": X1,
-        "flags2": X1,
+        "flags1": (
+            X1,
+            {
+                "combineTicks": U1,
+                "reserved3": U3,
+                "useWtSpeed": U1,
+                "dirPinPol": U1,
+                "useWtPin": U1,
+            },
+        ),
+        "flags2": (
+            X1,
+            {
+                "autoWtCountMaxOff": U1,
+                "autoDirPinPolOff": U1,
+                "autoSoftwareWtOff": U1,
+                "autoUseWtSpeedOff": U1,
+            },
+        ),
         "reserved1": U1,
         "wtFactor": U4,
         "wtQuantError": U4,
         "wtCountMax": U4,
         "wtLatency": U2,
         "wtFrequency": U1,
-        "flags3": X1,
+        "flags3": (
+            X1,
+            {
+                "reserved3": U4,
+                "cntBothEdges": U1,
+            },
+        ),
         "speedDeadBand": U2,
         "reserved2": U1,
     },
@@ -232,7 +310,13 @@ UBX_PAYLOADS_GET = {
             "numSources",
             {  # repeating group * numSources
                 "extInt": U1,
-                "flags": X2,
+                "flags": (
+                    X2,
+                    {
+                        "polarity": U1,
+                        "gnssUtc": U1,
+                    },
+                ),
                 "freq": U4,
                 "reserved2": U4,
                 "withTemp": U4,
@@ -278,8 +362,16 @@ UBX_PAYLOADS_GET = {
                 "gnssId": U1,
                 "resTrkCh": U1,
                 "maxTrkCh": U1,
-                "reserved1": U1,
-                "flags": X4,
+                "reserved0": U1,
+                "flags": (
+                    X4,
+                    {
+                        "enable": U1,
+                        "reserved1": U8,
+                        "reserved2": U7,
+                        "sigCfMask": U8,
+                    },
+                ),
             },
         ),
     },
@@ -289,18 +381,52 @@ UBX_PAYLOADS_GET = {
     },
     "CFG-INF": {
         "protocolID": U1,
-        "reserved1": U3,
-        "infMsgMaskDDC": X1,
-        "infMsgMaskUART1": X1,
-        "infMsgMaskUART2": X1,
-        "infMsgMaskUSB": X1,
-        "infMsgMaskSPI": X1,
-        "reserved2": X1,
+        "reserved0": U3,
+        "infMaskGroup": (
+            6,
+            {
+                "infMsgMask": (
+                    X1,
+                    {
+                        "enableError": U1,
+                        "enableWarning": U1,
+                        "enableNotice": U1,
+                        "enableTest": U1,
+                        "enableDebug": U1,
+                    },
+                ),
+            },
+        ),
+    },
+    "CFG-ITFM": {
+        "config": (
+            X4,
+            {
+                "bbThreshold": U4,
+                "cwThreshold": U5,
+                "algorithmBits": U22,
+                "enable": U1,
+            },
+        ),
+        "config2": (
+            X4,
+            {
+                "generalBits": U12,
+                "antSetting": U2,
+                "enable2": U1,
+            },
+        ),
     },
-    "CFG-ITFM": {"config": X4, "config2": X4},
     "CFG-LOGFILTER": {
         "version": U1,
-        "flags": X1,
+        "flags": (
+            X1,
+            {
+                "recordEnabled": U1,
+                "psmOncePerWakupEnabled": U1,
+                "applyAllFilterSettings": U1,
+            },
+        ),
         "minInterval": U2,
         "timeThreshold": U2,
         "speedThreshold": U2,
@@ -317,7 +443,22 @@ UBX_PAYLOADS_GET = {
         "reserved": U1,
     },
     "CFG-NAV5": {
-        "mask": X2,
+        "mask": (
+            X2,
+            {
+                "dyn": U1,
+                "minEl": U1,
+                "posFixMode": U1,
+                "drLim": U1,
+                "posMask": U1,
+                "timeMask": U1,
+                "staticHoldMask": U1,
+                "dgpsMask": U1,
+                "cnoThreshold": U1,
+                "reserved0": U1,
+                "utc": U1,
+            },
+        ),
         "dynModel": U1,
         "fixMode": U1,
         "fixedAlt": I4,
@@ -336,8 +477,30 @@ UBX_PAYLOADS_GET = {
     },
     "CFG-NAVX5": {
         "version": U2,
-        "mask1": X2,
-        "mask2": X4,
+        "mask1": (
+            X2,
+            {
+                "reserved9": U2,
+                "minMax": U1,
+                "minCno": U1,
+                "reserved10": U2,
+                "initial3dfix": U1,
+                "reserved11": U2,
+                "wknRoll": U1,
+                "ackAid": U1,
+                "reserved12": U2,
+                "ppp": U1,
+                "aop": U1,
+            },
+        ),
+        "mask2": (
+            X4,
+            {
+                "reserved13": U6,
+                "useAdr": U1,
+                "sigAttenComp": U1,
+            },
+        ),
         "reserved0": U2,
         "minSVs": U1,
         "maxSVs": U1,
@@ -360,28 +523,104 @@ UBX_PAYLOADS_GET = {
         "useAdr": U1,
     },
     "CFG-NMEAvX": {  # deprecated length 4
-        "filter": X1,
+        "filter": (
+            X1,
+            {
+                "posFilt": U1,
+                "mskPosFilt": U1,
+                "timeFilt": U1,
+                "dateFilt": U1,
+                "gpsOnlyFilter": U1,
+                "trackFilt": U1,
+            },
+        ),
         "nmeaVersion": U1,
         "numSV": U1,
-        "flags": X1,
+        "flags": (
+            X1,
+            {
+                "compat": U1,
+                "consider": U1,
+                "limit82": U1,
+                "highPrec": U1,
+            },
+        ),
     },
     "CFG-NMEAv0": {  # v0 deprecated length 12
-        "filter": X1,
+        "filter": (
+            X1,
+            {
+                "posFilt": U1,
+                "mskPosFilt": U1,
+                "timeFilt": U1,
+                "dateFilt": U1,
+                "gpsOnlyFilter": U1,
+                "trackFilt": U1,
+            },
+        ),
         "nmeaVersion": U1,
         "numSV": U1,
-        "flags": X1,
-        "gnssToFilter": X4,
+        "flags": (
+            X1,
+            {
+                "compat": U1,
+                "consider": U1,
+                "limit82": U1,
+                "highPrec": U1,
+            },
+        ),
+        "gnssToFilter": (
+            X4,
+            {
+                "disableGps": U1,
+                "disableSbas": U1,
+                "disableGalileo": U1,
+                "reserved2": U1,
+                "disableQzss": U1,
+                "disableGlonass": U1,
+                "disableBeidou": U1,
+            },
+        ),
         "svNumbering": U1,
         "mainTalkerId": U1,
         "gsvTalkerId": U1,
         "version": U1,
     },
     "CFG-NMEA": {  # preferred version length 20
-        "filter": X1,
+        "filter": (
+            X1,
+            {
+                "posFilt": U1,
+                "mskPosFilt": U1,
+                "timeFilt": U1,
+                "dateFilt": U1,
+                "gpsOnlyFilter": U1,
+                "trackFilt": U1,
+            },
+        ),
         "nmeaVersion": U1,
         "numSV": U1,
-        "flags": X1,
-        "gnssToFilter": X4,
+        "flags": (
+            X1,
+            {
+                "compat": U1,
+                "consider": U1,
+                "limit82": U1,
+                "highPrec": U1,
+            },
+        ),
+        "gnssToFilter": (
+            X4,
+            {
+                "disableGps": U1,
+                "disableSbas": U1,
+                "disableGalileo": U1,
+                "reserved2": U1,
+                "disableQzss": U1,
+                "disableGlonass": U1,
+                "disableBeidou": U1,
+            },
+        ),
         "svNumbering": U1,
         "mainTalkerId": U1,
         "gsvTalkerId": U1,
@@ -391,23 +630,52 @@ UBX_PAYLOADS_GET = {
     },
     "CFG-ODO": {
         "version": U1,
-        "reserved1": U3,
-        "flags": U1,
-        "odoCfg": X1,
-        "reserved2": U6,
+        "reserved0": U3,
+        "flags": (
+            X1,
+            {
+                "useODO": U1,
+                "useCOG": U1,
+                "outLPVel": U1,
+                "outLPCog": U1,
+            },
+        ),
+        "odoCfg": (
+            X1,
+            {
+                "profile": U3,
+            },
+        ),
+        "reserved1": U6,
         "cogMaxSpeed": U1,
         "cogMaxPosAcc": U1,
-        "reserved3": U2,
+        "reserved2": U2,
         "velLpGain": U1,
         "cogLpGain": U1,
-        "reserved4": U2,
+        "reserved3": U2,
     },
     "CFG-PM2": {
         "version": U1,
+        "reserved0": U1,
+        "maxStartupStateDur": U1,
         "reserved1": U1,
-        "reserved2": U1,
-        "reserved3": U1,
-        "flags": X4,
+        "flags": (
+            X4,
+            {
+                "optTarget": U3,
+                "extintSel": U1,
+                "extintWake": U1,
+                "extintBackup": U1,
+                "extintInactive": U1,
+                "limitPeakCurr": U2,
+                "waitTimeFix": U1,
+                "updateRTC": U1,
+                "updateEPH": U1,
+                "reserved12": U3,
+                "doNotEnterOff": U1,
+                "operationMode": U2,
+            },
+        ),
         "updatePeriod": U4,
         "searchPeriod": U4,
         "gridOffset": U4,
@@ -432,34 +700,168 @@ UBX_PAYLOADS_GET = {
     "CFG-PRT": {
         "portID": U1,
         "reserved0": U1,
-        "txReady": X2,
-        "mode": X4,
+        "txReady": (
+            X2,
+            {
+                "enable": U1,
+                "pol": U1,
+                "pin": U5,
+                "thres": U9,
+            },
+        ),
+        "UARTmode": (
+            X4,
+            {
+                "reserved6": U6,
+                "charLen": U2,
+                "reserved7": U1,
+                "parity": U3,
+                "nStopBits": U2,
+            },
+        ),
         "baudRate": U4,
-        "inProtoMask": X2,
-        "outProtoMask": X2,
+        "inProtoMask": (
+            X2,
+            {
+                "inUBX": U1,
+                "inNMEA": U1,
+                "inRTCM": U1,
+                "reserved8": U2,
+                "inRTCM3": U1,
+            },
+        ),
+        "outProtoMask": (
+            X2,
+            {
+                "outUBX": U1,
+                "outNMEA": U1,
+                "reserved9": U3,
+                "outRTCM3": U1,
+            },
+        ),
         "reserved4": U2,
         "reserved5": U2,
     },
     "CFG-PWR": {"version": U1, "reserved1": U3, "state": U4},
     "CFG-RATE": {"measRate": U2, "navRate": U2, "timeRef": U2},
-    "CFG-RINV": {"flags": X1, "group": ("None", {"data": U1})},  # repeating group
-    "CFG-RST": {"navBbrMask": X2, "resetMode": U1, "reserved1": U1},
-    "CFG-RXM": {"reserved1": U1, "lpMode": U1},
+    "CFG-RINV": {
+        "flags": (
+            X1,
+            {
+                "dump": U1,
+                "binary": U1,
+            },
+        ),
+        "group": ("None", {"data": U1}),
+    },  # repeating group
+    "CFG-RST": {
+        "navBbrMask": (
+            X2,
+            {
+                "eph": U1,
+                "alm": U1,
+                "health": U1,
+                "klob": U1,
+                "pos": U1,
+                "clkd": U1,
+                "osc": U1,
+                "utc": U1,
+                "rtc": U1,
+                "reserved2": U6,
+                "aop": U1,
+            },
+        ),
+        "resetMode": U1,
+        "reserved1": U1,
+    },
+    "CFG-RXM": {"reserved0": U1, "lpMode": U1},
     "CFG-SBAS": {
-        "mode": X1,
-        "usage": X1,
+        "mode": (
+            X1,
+            {
+                "enabled": U1,
+                "test": U1,
+            },
+        ),
+        "usage": (
+            X1,
+            {
+                "range": U1,
+                "diffCorr": U1,
+                "integrity": U1,
+            },
+        ),
         "maxSBAS": U1,
-        "scanmode2": X1,
-        "scanmode1": X4,
+        "scanmode2": (
+            X1,
+            {
+                "PRN152": U1,
+                "PRN153": U1,
+                "PRN154": U1,
+                "PRN155": U1,
+                "PRN156": U1,
+                "PRN157": U1,
+                "PRN158": U1,
+            },
+        ),
+        "scanmode1": (
+            X4,
+            {
+                "PRN120": U1,
+                "PRN121": U1,
+                "PRN122": U1,
+                "PRN123": U1,
+                "PRN124": U1,
+                "PRN125": U1,
+                "PRN126": U1,
+                "PRN127": U1,
+                "PRN128": U1,
+                "PRN129": U1,
+                "PRN130": U1,
+                "PRN131": U1,
+                "PRN132": U1,
+                "PRN133": U1,
+                "PRN134": U1,
+                "PRN135": U1,
+                "PRN136": U1,
+                "PRN137": U1,
+                "PRN138": U1,
+                "PRN139": U1,
+                "PRN140": U1,
+                "PRN141": U1,
+                "PRN142": U1,
+                "PRN143": U1,
+                "PRN144": U1,
+                "PRN145": U1,
+                "PRN146": U1,
+                "PRN147": U1,
+                "PRN148": U1,
+                "PRN149": U1,
+                "PRN150": U1,
+                "PRN151": U1,
+            },
+        ),
     },
     "CFG-SENIF": {
         "type": U1,
         "version": U1,
-        "flags": X1,
+        "flags": (
+            X2,
+            {
+                "senConn": U1,
+            },
+        ),
         "pioConf": X2,
     },
     "CFG-SLAS": {
-        "mode": X1,
+        "mode": (
+            X1,
+            {
+                "enabled": U1,
+                "test": U1,
+                "raim": U1,
+            },
+        ),
         "reserved1": U3,
     },
     "CFG-SMGR": {
@@ -469,9 +871,36 @@ UBX_PAYLOADS_GET = {
         "reserved1": U2,
         "freqTolerance": U2,
         "timeTolerance": U2,
-        "messageCfg": X2,
+        "messageCfg": (
+            X2,
+            {
+                "measInternal": U1,
+                "measGNSS": U1,
+                "measEXTINT0": U1,
+                "measEXTINT1": U1,
+            },
+        ),
         "maxSlewRate": U2,
-        "flags": X4,
+        "flags": (
+            X4,
+            {
+                "disableInternal": U1,
+                "disableExternal": U1,
+                "preferenceMode": U1,
+                "enableGNSS": U1,
+                "enableEXTINT0": U1,
+                "enableEXTINT1": U1,
+                "enableHostMeasInt": U1,
+                "enableHostMeasExt": U1,
+                "reserved1": U2,
+                "useAnyFix": U1,
+                "disableMaxSlewRate": U1,
+                "issueFreqWarning": U1,
+                "issueTimeWarning": U1,
+                "TPCoherent": U2,
+                "disableOffset": U1,
+            },
+        ),
     },
     "CFG-SPT": {
         "version": U1,
@@ -482,7 +911,13 @@ UBX_PAYLOADS_GET = {
     "CFG-TMODE2": {
         "timeMode": U1,
         "reserved1": U1,
-        "flags": X2,
+        "flags": (
+            X2,
+            {
+                "lla": U1,
+                "altInv": U1,
+            },
+        ),
         "ecefXOrLat": I4,
         "ecefYOrLon": I4,
         "ecefZOrAlt": I4,
@@ -493,7 +928,13 @@ UBX_PAYLOADS_GET = {
     "CFG-TMODE3": {
         "version": U1,
         "reserved0": U1,
-        "flags": X2,
+        "flags": (
+            X2,
+            {
+                "rcvrMode": U8,
+                "lla": U1,
+            },
+        ),
         "ecefXOrLat": I4,
         "ecefYOrLon": I4,
         "ecefZOrAlt": I4,
@@ -517,11 +958,33 @@ UBX_PAYLOADS_GET = {
         "pulseLenRatio": U4,
         "pulseLenRatioLock": U4,
         "userConfigDelay": I4,
-        "flags": X4,
+        "flags": (
+            X4,
+            {
+                "active": U1,
+                "lockGnssFreq": U1,
+                "lockedOtherSet": U1,
+                "isFreq": U1,
+                "isLength": U1,
+                "alignToTow": U1,
+                "polarity": U1,
+                "gridUtcGnss": U4,
+                "syncMode": U3,
+            },
+        ),
     },
     "CFG-TXSLOT": {
         "version": U1,
-        "enable": X1,
+        "enable": (
+            X1,
+            {
+                "enableDDC": U1,
+                "enableUART1": U1,
+                "enableUART2": U1,
+                "enableUSB": U1,
+                "enableSPI": U1,
+            },
+        ),
         "refTp": U1,
         "reserved1": U1,
         "end1": U4,
@@ -534,7 +997,13 @@ UBX_PAYLOADS_GET = {
         "reserved1": U2,
         "reserved2": U2,
         "powerConsumpt": U2,
-        "flags": X2,
+        "flags": (
+            X2,
+            {
+                "reEnum": U1,
+                "powerMode": U1,
+            },
+        ),
         "vendorString": C32,
         "productString": C32,
         "serialNumber": C32,
@@ -831,7 +1300,13 @@ UBX_PAYLOADS_GET = {
     "MON-COMMS": {
         "version": U1,
         "nPorts": U1,
-        "txErrors": X1,
+        "txErrors": (
+            X1,
+            {
+                "txErrorMem": U1,
+                "txErrorAlloc": U1,
+            },
+        ),
         "reserved0": U1,
         "protIds1": U1,
         "protIds2": U1,
@@ -861,9 +1336,33 @@ UBX_PAYLOADS_GET = {
     },
     "MON-GNSS": {
         "version": U1,
-        "supported": X1,
-        "default": X1,
-        "enabled": X1,
+        "supported": (
+            X1,
+            {
+                "GPSSup": U1,
+                "GlonassSup": U1,
+                "BeidouSup": U1,
+                "GalileoSup": U1,
+            },
+        ),
+        "default": (
+            X1,
+            {
+                "GPSDef": U1,
+                "GlonassDef": U1,
+                "BeidouDef": U1,
+                "GalileoDef": U1,
+            },
+        ),
+        "enabled": (
+            X1,
+            {
+                "GPSEna": U1,
+                "GlonassEna": U1,
+                "BeidouEna": U1,
+                "GalileoEna": U1,
+            },
+        ),
         "simultaneous": U1,
         "reserved1": U3,
     },
@@ -876,7 +1375,15 @@ UBX_PAYLOADS_GET = {
         "agcCnt": U2,
         "aStatus": U1,
         "aPower": U1,
-        "flags": X1,
+        "flags": (
+            X1,
+            {
+                "rtcCalib": U1,
+                "safeBoot": U1,
+                "jammingState": U2,
+                "xtalAbsent": U1,
+            },
+        ),
         "reserved1": U1,
         "usedMask": X4,
         "groupVP": (
@@ -898,20 +1405,39 @@ UBX_PAYLOADS_GET = {
         "magQ": U1,
         "cfgSource": U1,
         "reserved0": U3,
-        "lowLevCfg": X4,
+        "lowLevCfg": U4,
         "reserved11": U4,
         "reserved12": U4,
-        "postStatus": X4,
+        "postStatus": U4,
         "reserved2": U4,
     },
     "MON-HW3": {
         "version": U1,
         "nPins": U1,
-        "flags": X1,
+        "flags": (
+            X1,
+            {
+                "rtcCalib": U1,
+                "safeBoot": U1,
+                "xtalAbsent": U1,
+            },
+        ),
         "hwVersion": C10,
         "reserved0": U9,
         "pinId": U2,
-        "pinMask": X2,
+        "pinMask": (
+            X2,
+            {
+                "periphPIO": U1,
+                "pinBank": U3,
+                "direction": U1,
+                "pinValue": U1,
+                "vpManager": U1,
+                "pioIrq": U1,
+                "pioPullHigh": U1,
+                "pioPullLow": U1,
+            },
+        ),
         "VP": U1,
         "reserved1": U1,
     },
@@ -976,7 +1502,13 @@ UBX_PAYLOADS_GET = {
         "group": (
             "nEntries",
             {  # repeating group * nEntries
-                "patchInfo": X4,
+                "patchInfo": (
+                    X4,
+                    {
+                        "patchActivated": U1,
+                        "patchLocation": U2,
+                    },
+                ),
                 "comparatorNumber": U4,
                 "patchAddress": U4,
                 "patchData": U4,
@@ -991,7 +1523,12 @@ UBX_PAYLOADS_GET = {
             "nBlocks",
             {  # repeating group * nBlocks
                 "blockId": U1,
-                "flags": X1,
+                "flags": (
+                    X1,
+                    {
+                        "jammingState": U1,
+                    },
+                ),
                 "antStatus": U1,
                 "antPower": U1,
                 "postStatus": U4,
@@ -1027,17 +1564,57 @@ UBX_PAYLOADS_GET = {
             },
         ),  # repeating group * 6
     },
-    "MON-RXR": {"flags": U1},
+    "MON-RXR": {
+        "flags": (
+            X1,
+            {
+                "awake": U1,
+            },
+        ),
+    },
     "MON-SMGR": {
         "version": U1,
         "reserved1": U3,
         "iTOW": U4,
-        "intOsc": X2,
-        "extOsc": X2,
+        "intOsc": (
+            X2,
+            {
+                "intOscState": U4,
+                "intOscCalib": U1,
+                "intOscDisc": U1,
+            },
+        ),
+        "extOsc": (
+            X2,
+            {
+                "extscState": U4,
+                "extOscCalib": U1,
+                "extOscDisc": U1,
+            },
+        ),
         "discSrc": U1,
-        "gnss": X1,
-        "extInt0": X1,
-        "extInt1": X1,
+        "gnss": (
+            X1,
+            {
+                "gnssAvail": U1,
+            },
+        ),
+        "extInt0": (
+            X1,
+            {
+                "extInt0Avail": U1,
+                "extInt0Type": U1,
+                "extInt0FeedBack": U1,
+            },
+        ),
+        "extInt1": (
+            X1,
+            {
+                "extInt1Avail": U1,
+                "extInt1Type": U1,
+                "extInt1FeedBack": U1,
+            },
+        ),
     },
     "MON-SPAN": {
         "version": U1,
@@ -1046,12 +1623,7 @@ UBX_PAYLOADS_GET = {
         "group": (
             "numRfBlocks",
             {  # repeating group * numRfBlocks
-                "groupSpan": (
-                    256,
-                    {
-                        "spectrum": U1,
-                    },
-                ),  # nested repeating group * 256
+                "spectrum": A256,  # parsed as array of 256 integers
                 "span": U4,
                 "res": U4,
                 "center": U4,
@@ -1106,7 +1678,14 @@ UBX_PAYLOADS_GET = {
         ),  # repeating group * 6
         "tUsage": U1,
         "tPeakUsage": U1,
-        "errors": X1,
+        "errors": (
+            X1,
+            {
+                "txErrorLimit": U6,
+                "txErrorMem": U1,
+                "txErrorAlloc": U1,
+            },
+        ),
         "reserved1": U1,
     },
     "MON-VER": {
@@ -1232,7 +1811,12 @@ UBX_PAYLOADS_GET = {
     "NAV-HPPOSLLH": {
         "version": U1,
         "reserved": U2,
-        "flags": X1,
+        "flags": (
+            X1,
+            {
+                "invalidLlh": U1,
+            },
+        ),
         "iTOW": U4,
         "lon": I4,
         "lat": I4,
@@ -1249,13 +1833,66 @@ UBX_PAYLOADS_GET = {
         "iTOW": U4,
         "version": U1,
         "reserved1": U4,
-        "gpsNmiFlags": X1,
-        "gpsLsFlags": X1,
-        "galNmiFlags": X1,
-        "galLsFlags": X1,
-        "bdsNmiFlags": X1,
-        "bdsLsFlags": X1,
-        "gloNmiFlags": X1,
+        "gpsNmiFlags": (
+            X1,
+            {
+                "wnoCheckedGPS": U1,
+                "wnoInvalidGPS": U1,
+                "UTCORefCheckedGPS": U1,
+                "UTCORefInvalidGPS": U1,
+            },
+        ),
+        "gpsLsFlags": (
+            X1,
+            {
+                "lsValGPS": U1,
+                "dnRangeGPS": U1,
+                "totRangeGPS": U1,
+                "lsEventGPS": U1,
+                "recNowGPS": U1,
+            },
+        ),
+        "galNmiFlags": (
+            X1,
+            {
+                "wnoCheckedGAL": U1,
+                "wnoInvalidGAL": U1,
+            },
+        ),
+        "galLsFlags": (
+            X1,
+            {
+                "lsValGAL": U1,
+                "dnRangeGAL": U1,
+                "totRangeGAL": U1,
+                "lsEventGAL": U1,
+                "recNowGAL": U1,
+            },
+        ),
+        "bdsNmiFlags": (
+            X1,
+            {
+                "wnoCheckedBDS": U1,
+                "wnoInvalidBDS": U1,
+            },
+        ),
+        "bdsLsFlags": (
+            X1,
+            {
+                "lsValBDS": U1,
+                "dnRangeBDS": U1,
+                "totRangeBDS": U1,
+                "lsEventBDS": U1,
+                "recNowBDS": U1,
+            },
+        ),
+        "gloNmiFlags": (
+            X1,
+            {
+                "wnoCheckedGLO": U1,
+                "wnoInvalidGLO": U1,
+            },
+        ),
     },
     "NAV-ODO": {
         "version": U1,
@@ -1275,10 +1912,34 @@ UBX_PAYLOADS_GET = {
             {  # repeating group * numSv
                 "gnssId": U1,
                 "svId": U1,
-                "svFlag": X1,
-                "eph": X1,
-                "alm": X1,
-                "otherOrb": X1,
+                "svFlag": (
+                    X1,
+                    {
+                        "health": U2,
+                        "visibility": U2,
+                    },
+                ),
+                "eph": (
+                    X1,
+                    {
+                        "ephUsability": U5,
+                        "ephSource": U3,
+                    },
+                ),
+                "alm": (
+                    X1,
+                    {
+                        "almUsability": U5,
+                        "almSource": U3,
+                    },
+                ),
+                "otherOrb": (
+                    X1,
+                    {
+                        "anoAopUsability": U5,
+                        "type": U3,
+                    },
+                ),
             },
         ),
     },
@@ -1300,12 +1961,38 @@ UBX_PAYLOADS_GET = {
         "hour": U1,
         "min": U1,
         "second": U1,
-        "valid": X1,
+        "valid": (
+            X1,
+            {
+                "validDate": U1,
+                "validTime": U1,
+                "fullyResolved": U1,
+                "validMag": U1,
+                "reserved": U4,
+            },
+        ),
         "tAcc": U4,
         "nano": I4,
         "fixType": U1,
-        "flags": X1,
-        "flags2": X1,
+        "flags": (
+            X1,
+            {
+                "gnssFixOk": U1,
+                "difSoln": U1,
+                "psmState": U3,
+                "headVehValid": U1,
+                "carrSoln": U2,
+            },
+        ),
+        "flags2": (
+            X1,
+            {
+                "reserved": U5,
+                "confirmedAvai": U1,
+                "confirmedDate": U1,
+                "confirmedTime": U1,
+            },
+        ),
         "numSV": U1,
         "lon": I4,
         "lat": I4,
@@ -1347,7 +2034,20 @@ UBX_PAYLOADS_GET = {
         "accLength": U4,
         "accHeading": U4,
         "reserved2": U4,
-        "flags": X4,
+        "flags": (
+            X4,
+            {
+                "gnssFixOK": U1,
+                "diffSoln": U1,
+                "relPosValid": U1,
+                "carrSoln": U2,
+                "isMoving": U1,
+                "refPosMiss": U1,
+                "refObsMiss": U1,
+                "relPosHeadingValid": U1,
+                "relPosNormalized": U1,
+            },
+        ),
     },
     "NAV-SAT": {
         "iTOW": U4,
@@ -1364,7 +2064,28 @@ UBX_PAYLOADS_GET = {
                 "elev": I1,
                 "azim": I2,
                 "prRes": I2,
-                "flags": X4,
+                "flags": (
+                    X4,
+                    {
+                        "qualityInd": U3,
+                        "svUsed": U1,
+                        "health": U2,
+                        "diffCorr": U1,
+                        "smoothed": U1,
+                        "orbitSource": U3,
+                        "ephAvail": U1,
+                        "almAvail": U1,
+                        "anoAvail": U1,
+                        "aopAvail": U1,
+                        "sbasCorrUsed": U1,
+                        "rtcmCorrUsed": U1,
+                        "slasCorrUsed": U1,
+                        "spartnCorrUsed": U1,
+                        "prCorrUsed": U1,
+                        "crCorrUsed": U1,
+                        "doCorrUsed": U1,
+                    },
+                ),
             },
         ),
     },
@@ -1373,9 +2094,24 @@ UBX_PAYLOADS_GET = {
         "geo": U1,
         "mode": U1,
         "sys": I1,
-        "service": X1,
+        "service": (
+            X1,
+            {
+                "Ranging": U1,
+                "Corrections": U1,
+                "Integrity": U1,
+                "Testmode": U1,
+                "Bad": U1,
+            },
+        ),
         "numCh": U1,
-        "reserved0": U3,
+        "statusFlags": (
+            X1,
+            {
+                "integrityUsed": U2,
+            },
+        ),
+        "reserved0": U2,
         "channels": (
             "numCh",
             {  # repeating group * numCh
@@ -1408,7 +2144,19 @@ UBX_PAYLOADS_GET = {
                 "qualityInd": U1,
                 "corrSource": U1,
                 "ionoModel": U1,
-                "sigFlags": X2,
+                "sigFlags": (
+                    X2,
+                    {
+                        "health": U2,
+                        "prSmoothed": U1,
+                        "prUsed": U1,
+                        "crUsed": U1,
+                        "doUsed": U1,
+                        "prCorrUsed": U1,
+                        "crCorrUsed": U1,
+                        "doCorrUsed": U1,
+                    },
+                ),
                 "reserved1": U4,
             },
         ),
@@ -1421,7 +2169,14 @@ UBX_PAYLOADS_GET = {
         "gmsLat": I4,
         "gmsCode": U1,
         "qzssSvId": U1,
-        "serviceFlags": X1,
+        "serviceFlags": (
+            X1,
+            {
+                "gmsAvailable": U1,
+                "qzssSvAvailable": U1,
+                "testMode": U1,
+            },
+        ),
         "cnt": U1,
         "group": (
             "cnt",
@@ -1439,7 +2194,7 @@ UBX_PAYLOADS_GET = {
         "fTOW": I4,
         "week": I2,
         "gpsFix": U1,
-        "flags": X1,
+        "flags": X1,  # TODO
         "ecefX": I4,
         "ecefY": I4,
         "ecefZ": I4,
@@ -1456,24 +2211,71 @@ UBX_PAYLOADS_GET = {
     "NAV-STATUS": {
         "iTOW": U4,
         "gpsFix": U1,
-        "flags": X1,
-        "fixStat": X1,
-        "flags2": X1,
+        "flags": (
+            X1,
+            {
+                "gpsFixOk": U1,
+                "diffSoln": U1,
+                "wknSet": U1,
+                "towSet": U1,
+            },
+        ),
+        "fixStat": (
+            X1,
+            {
+                "diffCorr": U1,
+                "carrSolnValid": U1,
+                "reserved0": U4,
+                "mapMatching": U2,
+            },
+        ),
+        "flags2": (
+            X1,
+            {
+                "psmState": U2,
+                "reserved1": U1,
+                "spoofDetState": U2,
+                "reserved2": U1,
+                "carrSoln": U2,
+            },
+        ),
         "ttff": U4,
         "msss": U4,
     },
     "NAV-SVINFO": {  # deprecated - use NAV-SAT
         "iTOW": U4,
         "numCh": U1,
-        "globalFlags": X1,
+        "globalFlags": (
+            X1,
+            {
+                "chipGen": U3,
+            },
+        ),
         "reserved2": U2,
         "channels": (
             "numCh",
             {  # repeating group * numCh
                 "chn": U1,
                 "svid": U1,
-                "flags": X1,
-                "quality": X1,
+                "flags": (
+                    X1,
+                    {
+                        "svUsed": U1,
+                        "diffCorr": U1,
+                        "orbitAvail": U1,
+                        "orbitEph": U1,
+                        "unhealthy": U1,
+                        "orbitAlm": U1,
+                        "orbitAop": U1,
+                        "smoothed": U1,
+                    },
+                ),
+                "quality": (
+                    X1,
+                    {
+                        "qualityInd": U4,
+                    },
+                ),
                 "cno": U1,
                 "elev": I1,
                 "azim": I2,
@@ -1505,7 +2307,14 @@ UBX_PAYLOADS_GET = {
         "fSOW": I4,
         "week": I2,
         "leapS": I1,
-        "valid": X1,
+        "valid": (
+            X1,
+            {
+                "sowValid": U1,
+                "weekValid": U1,
+                "leapSValid": U1,
+            },
+        ),
         "tAcc": U4,
     },
     "NAV-TIMEGAL": {
@@ -1514,7 +2323,14 @@ UBX_PAYLOADS_GET = {
         "fGalTow": I4,
         "galWno": I2,
         "leapS": I1,
-        "valid": X1,
+        "valid": (
+            X1,
+            {
+                "galTowValid": U1,
+                "galWnoValid": U1,
+                "leapSValid": U1,
+            },
+        ),
         "tAcc": U4,
     },
     "NAV-TIMEGLO": {
@@ -1523,7 +2339,13 @@ UBX_PAYLOADS_GET = {
         "fTOD": I4,
         "Nt": U2,
         "N4": U1,
-        "valid": X1,
+        "valid": (
+            X1,
+            {
+                "todValid": U1,
+                "dateValid": U1,
+            },
+        ),
         "tAcc": U4,
     },
     "NAV-TIMEGPS": {
@@ -1531,7 +2353,14 @@ UBX_PAYLOADS_GET = {
         "fTOW": I4,
         "week": I2,
         "leapS": I1,
-        "valid": X1,
+        "valid": (
+            X1,
+            {
+                "towValid": U1,
+                "weekValid": U1,
+                "leapSValid": U1,
+            },
+        ),
         "tAcc": U4,
     },
     "NAV-TIMELS": {
@@ -1546,7 +2375,13 @@ UBX_PAYLOADS_GET = {
         "dateOfLsGpsWn": U2,
         "dateOfLsGpsDn": U2,
         "reserved2": U3,
-        "valid": X1,
+        "valid": (
+            X1,
+            {
+                "validCurrLs": U1,
+                "validTimeToLsEvent": U1,
+            },
+        ),
     },
     "NAV-TIMEQZSS": {
         "iTOW": U4,
@@ -1554,7 +2389,14 @@ UBX_PAYLOADS_GET = {
         "fQzssTow": I4,
         "qzssWno": I2,
         "leapS": I1,
-        "valid": X1,
+        "valid": (
+            X1,
+            {
+                "qzssTowValid": U1,
+                "qzssWnoValid": U1,
+                "leapSValid": U1,
+            },
+        ),
         "tAcc": U4,
     },
     "NAV-TIMEUTC": {
@@ -1567,7 +2409,16 @@ UBX_PAYLOADS_GET = {
         "hour": U1,
         "min": U1,
         "sec": U1,
-        "validflags": X1,
+        "validflags": (
+            X1,
+            {
+                "validTOW": U1,
+                "validWKN": U1,
+                "validUTC": U1,
+                "reserved0": U1,
+                "utcStandard": U4,
+            },
+        ),
     },
     "NAV-VELECEF": {"iTOW": U4, "ecefVX": I4, "ecefVY": I4, "ecefVZ": I4, "sAcc": U4},
     "NAV-VELNED": {
