@@ -3,12 +3,15 @@
 UBX Protocol Polling payload definitions
 
 THESE ARE THE PAYLOAD DEFINITIONS FOR _POLL_ MESSAGES _TO_ THE RECEIVER
+(e.g. query configuration; request monitoring, receiver management, logging or sensor fusion status)
 Response payloads are defined in UBX_PAYLOADS_GET
 
 NB: Attribute names must be unique within each message class/id
 
 Created on 27 Sep 2020
 
+Information sourced from u-blox Interface Specifications © 2013-2021, u-blox AG
+
 :author: semuadmin
 """
 # pylint: disable=too-many-lines, line-too-long
