@@ -3,6 +3,7 @@
 UBX Protocol Input payload definitions
 
 THESE ARE THE PAYLOAD DEFINITIONS FOR _SET_ MESSAGES _TO_ THE RECEIVER
+(e.g. configuration and calibration commands; AssistNow payloads)
 
 NB: Attribute names must be unique within each message class/id
 
@@ -20,6 +21,8 @@ NB: Repeating groups must be defined as a tuple thus:
 
 Created on 27 Sep 2020
 
+Information sourced from u-blox Interface Specifications © 2013-2021, u-blox AG
+
 :author: semuadmin
 """
 # pylint: disable=too-many-lines, line-too-long
@@ -37,10 +40,13 @@ from pyubx2.ubxtypes_core import (
     U2,
     U3,
     U4,
+    U5,
     U6,
     U7,
     U8,
     U9,
+    U12,
+    U22,
     U40,
     U64,
     X1,
@@ -128,17 +134,63 @@ UBX_PAYLOADS_SET = {
     # Messages in the CFG class are used to configure the receiver and read out current configuration values. Any
     # messages in the CFG class sent to the receiver are either acknowledged (with message UBX-ACK-ACK) if
     # processed successfully or rejected (with message UBX-ACK-NAK) if processing unsuccessfully.
-    "CFG-ANT": {"flags": X2, "pins": X2},
+    "CFG-ANT": {
+        "flags": (
+            X2,
+            {
+                "svcs": U1,
+                "scd": U1,
+                "ocd": U1,
+                "pdwnOnSCD": U1,
+                "recovery": U1,
+            },
+        ),
+        "pins": (
+            X2,
+            {
+                "pinSwitch": U5,
+                "pinSCD": U5,
+                "pinOCD": U5,
+                "reconfig": U1,
+            },
+        ),
+    },
     "CFG-BATCH": {
         "version": U1,
-        "flags": X1,
+        "flags": (
+            X1,
+            {
+                "enable": U1,
+                "reserved1": U1,
+                "extraPvt": U1,
+                "extraOdo": U1,
+                "reserved2": U1,
+                "pioEnable": U1,
+                "pioActiveLow": U1,
+            },
+        ),
         "bufSize": U2,
         "notifThrs": U2,
         "pioId": U1,
         "reserved0": U1,
     },
-    "CFG-CFG": {"clearMask": X4, "saveMask": X4, "loadMask": X4, "deviceMask": X1},
+    "CFG-CFG": {
+        "clearMask": X4,
+        "saveMask": X4,
+        "loadMask": X4,
+        "deviceMask": (
+            X1,
+            {
+                "devBBR": U1,
+                "devFlash": U1,
+                "devEEPROM": U1,
+                "devSpiFlash": U1,
+            },
+        ),
+    },
     "CFG-DAT": {
+        "datumNum": U2,
+        "datumName": C6,
         "majA": R8,
         "flat": R8,
         "dX": R4,
@@ -162,7 +214,13 @@ UBX_PAYLOADS_SET = {
             {  # repeating group * numOsc
                 "oscId": U1,
                 "reserved2": U1,
-                "flags": X2,
+                "flags": (
+                    X2,
+                    {
+                        "isCalibrated": U1,
+                        "controlIf": U4,
+                    },
+                ),
                 "freq": U4,
                 "phaseOffset": I4,
                 "withTemp": U4,
@@ -203,15 +261,38 @@ UBX_PAYLOADS_SET = {
     },
     "CFG-ESFWT": {
         "version": U1,
-        "flags1": X1,
-        "flags2": X1,
+        "flags1": (
+            X1,
+            {
+                "combineTicks": U1,
+                "reserved3": U3,
+                "useWtSpeed": U1,
+                "dirPinPol": U1,
+                "useWtPin": U1,
+            },
+        ),
+        "flags2": (
+            X1,
+            {
+                "autoWtCountMaxOff": U1,
+                "autoDirPinPolOff": U1,
+                "autoSoftwareWtOff": U1,
+                "autoUseWtSpeedOff": U1,
+            },
+        ),
         "reserved1": U1,
         "wtFactor": U4,
         "wtQuantError": U4,
         "wtCountMax": U4,
         "wtLatency": U2,
         "wtFrequency": U1,
-        "flags3": X1,
+        "flags3": (
+            X1,
+            {
+                "reserved3": U4,
+                "cntBothEdges": U1,
+            },
+        ),
         "speedDeadBand": U2,
         "reserved2": U1,
     },
@@ -223,7 +304,13 @@ UBX_PAYLOADS_SET = {
             "numSources",
             {  # repeating group * numSources
                 "extInt": U1,
-                "flags": X2,
+                "flags": (
+                    X2,
+                    {
+                        "polarity": U1,
+                        "gnssUtc": U1,
+                    },
+                ),
                 "freq": U4,
                 "reserved2": U4,
                 "withTemp": U4,
@@ -269,8 +356,16 @@ UBX_PAYLOADS_SET = {
                 "gnssId": U1,
                 "resTrkCh": U1,
                 "maxTrkCh": U1,
-                "reserved1": U1,
-                "flags": X4,
+                "reserved0": U1,
+                "flags": (
+                    X4,
+                    {
+                        "enable": U1,
+                        "reserved1": U8,
+                        "reserved2": U7,
+                        "sigCfMask": U8,
+                    },
+                ),
             },
         ),
     },
@@ -280,18 +375,52 @@ UBX_PAYLOADS_SET = {
     },
     "CFG-INF": {
         "protocolID": U1,
-        "reserved1": U3,
-        "infMsgMaskDDC": X1,
-        "infMsgMaskUART1": X1,
-        "infMsgMaskUART2": X1,
-        "infMsgMaskUSB": X1,
-        "infMsgMaskSPI": X1,
-        "reserved2": X1,
+        "reserved0": U3,
+        "infMaskGroup": (
+            6,
+            {
+                "infMsgMask": (
+                    X1,
+                    {
+                        "enableError": U1,
+                        "enableWarning": U1,
+                        "enableNotice": U1,
+                        "enableTest": U1,
+                        "enableDebug": U1,
+                    },
+                ),
+            },
+        ),
+    },
+    "CFG-ITFM": {
+        "config": (
+            X4,
+            {
+                "bbThreshold": U4,
+                "cwThreshold": U5,
+                "algorithmBits": U22,
+                "enable": U1,
+            },
+        ),
+        "config2": (
+            X4,
+            {
+                "generalBits": U12,
+                "antSetting": U2,
+                "enable2": U1,
+            },
+        ),
     },
-    "CFG-ITFM": {"config": X4, "config2": X4},
     "CFG-LOGFILTER": {
         "version": U1,
-        "flags": X1,
+        "flags": (
+            X1,
+            {
+                "recordEnabled": U1,
+                "psmOncePerWakupEnabled": U1,
+                "applyAllFilterSettings": U1,
+            },
+        ),
         "minInterval": U2,
         "timeThreshold": U2,
         "speedThreshold": U2,
@@ -308,7 +437,22 @@ UBX_PAYLOADS_SET = {
         "reserved": U1,
     },
     "CFG-NAV5": {
-        "mask": X2,
+        "mask": (
+            X2,
+            {
+                "dyn": U1,
+                "minEl": U1,
+                "posFixMode": U1,
+                "drLim": U1,
+                "posMask": U1,
+                "timeMask": U1,
+                "staticHoldMask": U1,
+                "dgpsMask": U1,
+                "cnoThreshold": U1,
+                "reserved0": U1,
+                "utc": U1,
+            },
+        ),
         "dynModel": U1,
         "fixMode": U1,
         "fixedAlt": I4,
@@ -327,8 +471,30 @@ UBX_PAYLOADS_SET = {
     },
     "CFG-NAVX5": {
         "version": U2,
-        "mask1": X2,
-        "mask2": X4,
+        "mask1": (
+            X2,
+            {
+                "reserved9": U2,
+                "minMax": U1,
+                "minCno": U1,
+                "reserved10": U2,
+                "initial3dfix": U1,
+                "reserved11": U2,
+                "wknRoll": U1,
+                "ackAid": U1,
+                "reserved12": U2,
+                "ppp": U1,
+                "aop": U1,
+            },
+        ),
+        "mask2": (
+            X4,
+            {
+                "reserved13": U6,
+                "useAdr": U1,
+                "sigAttenComp": U1,
+            },
+        ),
         "reserved0": U2,
         "minSVs": U1,
         "maxSVs": U1,
@@ -351,28 +517,104 @@ UBX_PAYLOADS_SET = {
         "useAdr": U1,
     },
     "CFG-NMEAvX": {  # deprecated length 4
-        "filter": X1,
+        "filter": (
+            X1,
+            {
+                "posFilt": U1,
+                "mskPosFilt": U1,
+                "timeFilt": U1,
+                "dateFilt": U1,
+                "gpsOnlyFilter": U1,
+                "trackFilt": U1,
+            },
+        ),
         "nmeaVersion": U1,
         "numSV": U1,
-        "flags": X1,
+        "flags": (
+            X1,
+            {
+                "compat": U1,
+                "consider": U1,
+                "limit82": U1,
+                "highPrec": U1,
+            },
+        ),
     },
     "CFG-NMEAv0": {  # v0 deprecated length 12
-        "filter": X1,
+        "filter": (
+            X1,
+            {
+                "posFilt": U1,
+                "mskPosFilt": U1,
+                "timeFilt": U1,
+                "dateFilt": U1,
+                "gpsOnlyFilter": U1,
+                "trackFilt": U1,
+            },
+        ),
         "nmeaVersion": U1,
         "numSV": U1,
-        "flags": X1,
-        "gnssToFilter": X4,
+        "flags": (
+            X1,
+            {
+                "compat": U1,
+                "consider": U1,
+                "limit82": U1,
+                "highPrec": U1,
+            },
+        ),
+        "gnssToFilter": (
+            X4,
+            {
+                "disableGps": U1,
+                "disableSbas": U1,
+                "disableGalileo": U1,
+                "reserved2": U1,
+                "disableQzss": U1,
+                "disableGlonass": U1,
+                "disableBeidou": U1,
+            },
+        ),
         "svNumbering": U1,
         "mainTalkerId": U1,
         "gsvTalkerId": U1,
         "version": U1,
     },
     "CFG-NMEA": {  # preferred version length 20
-        "filter": X1,
+        "filter": (
+            X1,
+            {
+                "posFilt": U1,
+                "mskPosFilt": U1,
+                "timeFilt": U1,
+                "dateFilt": U1,
+                "gpsOnlyFilter": U1,
+                "trackFilt": U1,
+            },
+        ),
         "nmeaVersion": U1,
         "numSV": U1,
-        "flags": X1,
-        "gnssToFilter": X4,
+        "flags": (
+            X1,
+            {
+                "compat": U1,
+                "consider": U1,
+                "limit82": U1,
+                "highPrec": U1,
+            },
+        ),
+        "gnssToFilter": (
+            X4,
+            {
+                "disableGps": U1,
+                "disableSbas": U1,
+                "disableGalileo": U1,
+                "reserved2": U1,
+                "disableQzss": U1,
+                "disableGlonass": U1,
+                "disableBeidou": U1,
+            },
+        ),
         "svNumbering": U1,
         "mainTalkerId": U1,
         "gsvTalkerId": U1,
@@ -382,23 +624,52 @@ UBX_PAYLOADS_SET = {
     },
     "CFG-ODO": {
         "version": U1,
-        "reserved1": U3,
-        "flags": U1,
-        "odoCfg": X1,
-        "reserved2": U6,
+        "reserved0": U3,
+        "flags": (
+            X1,
+            {
+                "useODO": U1,
+                "useCOG": U1,
+                "outLPVel": U1,
+                "outLPCog": U1,
+            },
+        ),
+        "odoCfg": (
+            X1,
+            {
+                "profile": U3,
+            },
+        ),
+        "reserved1": U6,
         "cogMaxSpeed": U1,
         "cogMaxPosAcc": U1,
-        "reserved3": U2,
+        "reserved2": U2,
         "velLpGain": U1,
         "cogLpGain": U1,
-        "reserved4": U2,
+        "reserved3": U2,
     },
     "CFG-PM2": {
         "version": U1,
+        "reserved0": U1,
+        "maxStartupStateDur": U1,
         "reserved1": U1,
-        "reserved2": U1,
-        "reserved3": U1,
-        "flags": X4,
+        "flags": (
+            X4,
+            {
+                "optTarget": U3,
+                "extintSel": U1,
+                "extintWake": U1,
+                "extintBackup": U1,
+                "extintInactive": U1,
+                "limitPeakCurr": U2,
+                "waitTimeFix": U1,
+                "updateRTC": U1,
+                "updateEPH": U1,
+                "reserved12": U3,
+                "doNotEnterOff": U1,
+                "operationMode": U2,
+            },
+        ),
         "updatePeriod": U4,
         "searchPeriod": U4,
         "gridOffset": U4,
@@ -423,34 +694,168 @@ UBX_PAYLOADS_SET = {
     "CFG-PRT": {
         "portID": U1,
         "reserved0": U1,
-        "txReady": X2,
-        "mode": X4,
+        "txReady": (
+            X2,
+            {
+                "enable": U1,
+                "pol": U1,
+                "pin": U5,
+                "thres": U9,
+            },
+        ),
+        "UARTmode": (
+            X4,
+            {
+                "reserved6": U6,
+                "charLen": U2,
+                "reserved7": U1,
+                "parity": U3,
+                "nStopBits": U2,
+            },
+        ),
         "baudRate": U4,
-        "inProtoMask": X2,
-        "outProtoMask": X2,
+        "inProtoMask": (
+            X2,
+            {
+                "inUBX": U1,
+                "inNMEA": U1,
+                "inRTCM": U1,
+                "reserved8": U2,
+                "inRTCM3": U1,
+            },
+        ),
+        "outProtoMask": (
+            X2,
+            {
+                "outUBX": U1,
+                "outNMEA": U1,
+                "reserved9": U3,
+                "outRTCM3": U1,
+            },
+        ),
         "reserved4": U2,
         "reserved5": U2,
     },
     "CFG-PWR": {"version": U1, "reserved1": U3, "state": U4},
     "CFG-RATE": {"measRate": U2, "navRate": U2, "timeRef": U2},
-    "CFG-RINV": {"flags": X1, "group": ("None", {"data": U1})},  # repeating group
-    "CFG-RST": {"navBbrMask": X2, "resetMode": U1, "reserved1": U1},
-    "CFG-RXM": {"reserved1": U1, "lpMode": U1},
+    "CFG-RINV": {
+        "flags": (
+            X1,
+            {
+                "dump": U1,
+                "binary": U1,
+            },
+        ),
+        "group": ("None", {"data": U1}),
+    },  # repeating group
+    "CFG-RST": {
+        "navBbrMask": (
+            X2,
+            {
+                "eph": U1,
+                "alm": U1,
+                "health": U1,
+                "klob": U1,
+                "pos": U1,
+                "clkd": U1,
+                "osc": U1,
+                "utc": U1,
+                "rtc": U1,
+                "reserved2": U6,
+                "aop": U1,
+            },
+        ),
+        "resetMode": U1,
+        "reserved1": U1,
+    },
+    "CFG-RXM": {"reserved0": U1, "lpMode": U1},
     "CFG-SBAS": {
-        "mode": X1,
-        "usage": X1,
+        "mode": (
+            X1,
+            {
+                "enabled": U1,
+                "test": U1,
+            },
+        ),
+        "usage": (
+            X1,
+            {
+                "range": U1,
+                "diffCorr": U1,
+                "integrity": U1,
+            },
+        ),
         "maxSBAS": U1,
-        "scanmode2": X1,
-        "scanmode1": X4,
+        "scanmode2": (
+            X1,
+            {
+                "PRN152": U1,
+                "PRN153": U1,
+                "PRN154": U1,
+                "PRN155": U1,
+                "PRN156": U1,
+                "PRN157": U1,
+                "PRN158": U1,
+            },
+        ),
+        "scanmode1": (
+            X4,
+            {
+                "PRN120": U1,
+                "PRN121": U1,
+                "PRN122": U1,
+                "PRN123": U1,
+                "PRN124": U1,
+                "PRN125": U1,
+                "PRN126": U1,
+                "PRN127": U1,
+                "PRN128": U1,
+                "PRN129": U1,
+                "PRN130": U1,
+                "PRN131": U1,
+                "PRN132": U1,
+                "PRN133": U1,
+                "PRN134": U1,
+                "PRN135": U1,
+                "PRN136": U1,
+                "PRN137": U1,
+                "PRN138": U1,
+                "PRN139": U1,
+                "PRN140": U1,
+                "PRN141": U1,
+                "PRN142": U1,
+                "PRN143": U1,
+                "PRN144": U1,
+                "PRN145": U1,
+                "PRN146": U1,
+                "PRN147": U1,
+                "PRN148": U1,
+                "PRN149": U1,
+                "PRN150": U1,
+                "PRN151": U1,
+            },
+        ),
     },
     "CFG-SENIF": {
         "type": U1,
         "version": U1,
-        "flags": X1,
+        "flags": (
+            X2,
+            {
+                "senConn": U1,
+            },
+        ),
         "pioConf": X2,
     },
     "CFG-SLAS": {
-        "mode": X1,
+        "mode": (
+            X1,
+            {
+                "enabled": U1,
+                "test": U1,
+                "raim": U1,
+            },
+        ),
         "reserved1": U3,
     },
     "CFG-SMGR": {
@@ -460,9 +865,36 @@ UBX_PAYLOADS_SET = {
         "reserved1": U2,
         "freqTolerance": U2,
         "timeTolerance": U2,
-        "messageCfg": X2,
+        "messageCfg": (
+            X2,
+            {
+                "measInternal": U1,
+                "measGNSS": U1,
+                "measEXTINT0": U1,
+                "measEXTINT1": U1,
+            },
+        ),
         "maxSlewRate": U2,
-        "flags": X4,
+        "flags": (
+            X4,
+            {
+                "disableInternal": U1,
+                "disableExternal": U1,
+                "preferenceMode": U1,
+                "enableGNSS": U1,
+                "enableEXTINT0": U1,
+                "enableEXTINT1": U1,
+                "enableHostMeasInt": U1,
+                "enableHostMeasExt": U1,
+                "reserved1": U2,
+                "useAnyFix": U1,
+                "disableMaxSlewRate": U1,
+                "issueFreqWarning": U1,
+                "issueTimeWarning": U1,
+                "TPCoherent": U2,
+                "disableOffset": U1,
+            },
+        ),
     },
     "CFG-SPT": {
         "version": U1,
@@ -473,7 +905,13 @@ UBX_PAYLOADS_SET = {
     "CFG-TMODE2": {
         "timeMode": U1,
         "reserved1": U1,
-        "flags": X2,
+        "flags": (
+            X2,
+            {
+                "lla": U1,
+                "altInv": U1,
+            },
+        ),
         "ecefXOrLat": I4,
         "ecefYOrLon": I4,
         "ecefZOrAlt": I4,
@@ -484,7 +922,13 @@ UBX_PAYLOADS_SET = {
     "CFG-TMODE3": {
         "version": U1,
         "reserved0": U1,
-        "flags": X2,
+        "flags": (
+            X2,
+            {
+                "rcvrMode": U8,
+                "lla": U1,
+            },
+        ),
         "ecefXOrLat": I4,
         "ecefYOrLon": I4,
         "ecefZOrAlt": I4,
@@ -508,11 +952,33 @@ UBX_PAYLOADS_SET = {
         "pulseLenRatio": U4,
         "pulseLenRatioLock": U4,
         "userConfigDelay": I4,
-        "flags": X4,
+        "flags": (
+            X4,
+            {
+                "active": U1,
+                "lockGnssFreq": U1,
+                "lockedOtherSet": U1,
+                "isFreq": U1,
+                "isLength": U1,
+                "alignToTow": U1,
+                "polarity": U1,
+                "gridUtcGnss": U4,
+                "syncMode": U3,
+            },
+        ),
     },
     "CFG-TXSLOT": {
         "version": U1,
-        "enable": X1,
+        "enable": (
+            X1,
+            {
+                "enableDDC": U1,
+                "enableUART1": U1,
+                "enableUART2": U1,
+                "enableUSB": U1,
+                "enableSPI": U1,
+            },
+        ),
         "refTp": U1,
         "reserved1": U1,
         "end1": U4,
@@ -525,7 +991,13 @@ UBX_PAYLOADS_SET = {
         "reserved1": U2,
         "reserved2": U2,
         "powerConsumpt": U2,
-        "flags": X2,
+        "flags": (
+            X2,
+            {
+                "reEnum": U1,
+                "powerMode": U1,
+            },
+        ),
         "vendorString": C32,
         "productString": C32,
         "serialNumber": C32,
